package com.example.game;

import java.util.ConcurrentModificationException;
import java.util.Timer;
import java.util.TimerTask;
import android.graphics.Canvas;
import android.view.SurfaceHolder;
import android.util.Log;

/**
 * Clase que implementa el hilo con el bucle de control y ejecucin del juego.
 * 
 * @author Antonio Javier Garrido Rodrguez
 * Copyright 2014 Antonio Javier Garrido Rodrguez
 * 
 *  This file is part of Kamikawa.

    Kamikawa is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Kamikawa is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Kamikawa.  If not, see <http://www.gnu.org/licenses/>.
 */

public class GameLoopThread extends Thread {
	
	private static final String Tag = GameLoopThread.class.getSimpleName();
    
	private GameView _view;
    private boolean _running;
    //Para indicar si el nivel ha sido finalizado por la superacin del tiempo
    private boolean _isFinishedLevel;
    //Indica si se ha perdido la partida por superar el lmite de agentes permitidos
    private boolean _isLostGame;
    //Indica los frames(ejecuciones del bucle) por segundos
    static long FPS;
    //Indica el nivel actual
    private int _level;
    //Indica el tiempo de creacin de agentes
    private int _timeMakeAgent;
    
    //Indica si el hilo est durmiendo
    private boolean _isSleeping;
    
	 //Referencia a la actividad principal
	 FrmStartGame _frmStartGame;
	 
	 //Contabiliza los segunos transcurridos en la partida
	 private int _countTime;
	 
	 //Contabiliza el tiempo para crear a los agentes
	 private long _countTimeToCreateAgent;
	 
	 //Utilizado como indicador para contabilizar tiempo o no
	 private boolean _mustCount;
	 
	 //Para controlar si iniciamos el nivel desde la pantalla o por superacin del anterior
	 private boolean _isLevelStartedFromScreen;
	 
	 //Para terminar la ejecucin del hilo
	 private boolean _destroy;
	 
	 //Indica el nmero de agentes a partir del cual perdemos la partida para el nivel actual
	 private int _numAgentToLost;
		
      
    /**
     * Inicializa los valores.
     * @param view
     */
    public GameLoopThread(GameView view) 
    {
    	//Inicializo variables
    	_running = false;
    	_isFinishedLevel = false;
    	_isLostGame = false;
    	FPS = 10;
    	_timeMakeAgent = 0;
    	_frmStartGame = null;
    	_countTime = 0;
    	_countTimeToCreateAgent = 0;
    	//Indicamos que debe contar
    	_mustCount = true;
    	
    	_numAgentToLost = 0;
    	
    	//Indicamos inicialmente que no est durmiendo
    	_isSleeping = false;
    	
    	//Ontengo referencia al SurfaceView
    	this._view = view;
        
        //Obtengo una referencia a la actividad
    	_frmStartGame = (FrmStartGame) this._view.getContext();  
          
	    //Programamos una tarea para que se ejecute cada segundo, as poder contabilizar el tiempo
        Timer timer = new Timer();
        timer.scheduleAtFixedRate(taskToCounterTheTime, 1000, 1000);
               
    }

    
	/**
	 * Tarea invocada cada segundo, para contabilizar el tiempo transcurrido y comprobar si el blucle de ejecucin debe parar
	 */
	private TimerTask taskToCounterTheTime = new TimerTask() 
	{   
		@Override
	    public void run() 
	    {   
	        //Si el juego no est pausado y el indicador de cuenta es true contabilizo el tiempo
	        if(! _frmStartGame.isPausedGame() && _mustCount)
	        {
	        	//Compruebo si el juego debe continuar o pararse, bien por superar el nivel o bien por perder la partida
	        	checkConditions();
	        	
	        	//Imprime en el TextView el contador de segundos
	        	printInTextViewTime();
	        	_countTime++;   
	        }
	    }
	};
    
    
    public void setRunning(boolean run) 
    {
          _running = run;      
    }
    
    public void setDestroy(Boolean destroy)
    {
    	_destroy = destroy;
    }
    	
    	
    /**
     * Establece valores de configuracin para la ejecucin. 
     */
    public void setConfig()
    {
    	_level = MyApplication.getCurrentLevel();
    	
    	//Obtengo el nmero de agentes a partir del cual perdemos la partida para el nivel actual
    	_numAgentToLost = MyApplication.Config.getNumAgentToLost(_level);
    	
    	//Obtengo dependiendo del nivel cada cuantos milisegundos debe crear a un agente
    	_timeMakeAgent = MyApplication.Config.getTimeMakeAgent(_level);
    	
        //Iniciamos nivel desde la pantalla de eleccin
        _isLevelStartedFromScreen = true;
    }
    
    
    /**
     * Inprime en pantalla el nivel que se iniciar, cuando se inicia desde la pantalla "niveles disponibles" 
     */
    private void startNewLevel()
    {
    	
    	//La pantalla an no se ha pintado, mando a pintarla
		updateScreen();
		
    	//Escribo el nuevo nivel en el textView
    	_frmStartGame.setTextViewNewLevel(String.valueOf(_level));
    	//Hago visible el nuevo nivel
    	_frmStartGame.setTextViewVisibleNewLevel();
    	goToSleep(2500);
    	
    	//Oculto el nuevo nivel
    	_frmStartGame.setTextViewInvisibleNewLevel();
    	
    	goToSleep(1000);
    	
		//Indico el nivel seleccionado
        _frmStartGame.setTextViewLevel(String.valueOf(_level));

        
		//Muestro el HUD
		_frmStartGame.showHUD();
		
		goToSleep(500);
		
    	//Reproduzco la msica ambiente
        MyApplication.getSoundManager().startPlayer();
        
    	//Indico que debe contabilizar tiempo de juego
    	_mustCount = true;
    	
    	//Habilito los controles de interaccin
    	_frmStartGame.enableControls();
              		
    }
    
    
    /** 
     * Implementa el bucle de ejecucin del juego.
     */
    @Override
    public void run() {
    	
    	//Si el hilo no debe destruirse
    	if(!_destroy)
		{
    	
	    	//Empezamos un nuevo nivel seleccionado desde la pantalla de inicio del juego
	    	if (_isLevelStartedFromScreen)
	    	{
	        	//Indico que no debe contabilizar tiempo de juego
	        	_mustCount = false;
	        	_isLevelStartedFromScreen = false;
	        	//Inhabilito los controles de interaccin
	        	_frmStartGame.disableControls();
	    		startNewLevel();
	    	}
	    	
	    	
	        long ticksPS = 1000 / FPS;
	        long startTime;
	        long sleepTime;
	        long lapsed = 0;
	       
	         
	        //Obtengo el primer agente que se debe generar
	        _view.prepareNextAgent();
				
			
			while (_running) {
				       
	        //Si el juego no est pausado que continue la actividad de los agentes
	        if(! _frmStartGame.isPausedGame())
	        {
	            
	            //Tiempo inicial de ejecucin 
	            startTime = System.currentTimeMillis();
	        	  	       	     	        	  
	        	//Compruebo el tiempo, para determinar si debo crear un agente
	        	if(_countTimeToCreateAgent >= _timeMakeAgent)
	        	{
	        		_view.createAgent();
	        		_countTimeToCreateAgent = 0;
	        	}
	        	 
	                          
	            //Indico que se dibuje la pantalla con todo su contenido
	        	updateScreen();
	              
	            //Tiempo transcurrido
	            lapsed = System.currentTimeMillis() - startTime;
	            sleepTime = ticksPS - lapsed;
	                 
	            //Tiempo real en completar una vuelta del bucle
	            _countTimeToCreateAgent = _countTimeToCreateAgent + lapsed;
	                 
	            //Si se ha completado una ejecucin en menos tiempo del establecido, que duerma hasta completar el tiemp establecido.  
	            if (sleepTime > 0)
	            {
	            	goToSleep(sleepTime);
	                //Contabilizamos el tiempo que duerme el proceso
	                _countTimeToCreateAgent = _countTimeToCreateAgent + sleepTime;
	            }
	            else
	            {
	            	goToSleep(10);
	                //Contabilizamos el tiempo que duerme el proceso
	                _countTimeToCreateAgent = _countTimeToCreateAgent + 10;
	             }
	              
	        }
	        		
	        }
			
	    	if(!_destroy)
			{
			
	    		//Si hemos superado el nivel
	    		if(_isFinishedLevel)
	    			finishedLevel();
	    		//Sino, hemos perdido la partida
	    		else if(_isLostGame)
	    			lostGame();
			}
		}
    }
     
    
    /**
     * Comprueba las condiciones de parada de ejecucin del bucle, es decir, si se ha superado el tiempo que dura un nivel
     * o hemos superado el lmite de agentes permitidos.
     */
    private void checkConditions()
    {
		//Si ha transcurrido el tiempo determinado que dura un nivel, indicamos el final del nivel
    	if(_countTime >= 120)
		{
			_running = false;
			_isFinishedLevel = true;
		}
    	//Compruebo si hemos llegado al nmero mximo de agentes permitidos en pantalla
    	else if(_view.listSpritesSize() >= _numAgentToLost )
    	{
    		_running = false;
    		_isLostGame = true;
    	}

    }
    
    /**
     * Mtodo ejecutado cuando se supera un nivel y debemos pasar al siguiente. 
     */
    private void finishedLevel()
	{
		//Compruebo si supero el ltimo nivel, en tal caso, hemos llegado al final del juego.
		if(_level == 10)
			winGame();
		else
		{
	    	MyApplication.getSoundManager().pausePlayer();
	    	
	    	//Pauso el contador del tiempo
	    	_mustCount = false;
	    	
	    	//Inhabilito los controles de interaccin
	    	_frmStartGame.disableControls();
	    	
	    	//Escribo el mensaje de nivel superado
	    	_frmStartGame.setTextViewFinishedLevel("Nivel completado");
	    	//Hago visible el mensaje de nivel superado
	        _frmStartGame.setTextViewVisibleFinishedLevel();
	        
	        updateScreen();
	        goToSleep(2000);
	        
	        MyApplication.getSoundManager().playSound(MyApplication.Sounds.Applause);
	    	
	        //Escribo los puntos totales en el textview que muestra los puntos
	        _frmStartGame.setTextViewPointsValue(String.valueOf(_view.getPointsValue()));
	        //Hago visible el textView 
	        _frmStartGame.setTextViewVisiblePoints();
	        
	        updateScreen();
	        goToSleep(3000);
	        
	        
	        //Inicialmente digo que las letras en pantalla son 0
	        _frmStartGame.setTextViewLettersLeftValue(String.valueOf(0));
	        //Hago visible el textView 
	        _frmStartGame.setTextViewVisibleLettersLeft();
	        
	        updateScreen();
	        goToSleep(3000);
	        
	        //Invoco al mtodo que cuenta y resta a la puntuacin total las letras que queden en pantalla.
	        //Devolviendo true si haba al menos una letra
	        if(counterLettersLeft())
	        	//Actualizo los puntos a su nuevo valor
	        	_frmStartGame.setTextViewPoints(String.valueOf(_view.getPointsValue()));
	        
	        goToSleep(1500);
	        
	        //Escribo los puntos totales una vez restado las letras en pantalla 
	        _frmStartGame.setTextViewTotalPointsValue(String.valueOf(_view.getPointsValue()));
	         //Hago visible el textView 
	        _frmStartGame.setTextViewVisibleTotalPoints();
	        
	        //Reseteo el Display
	        _view.resetDisplay();
	        
	        updateScreen();
	        goToSleep(4000);
	        		
	    	//Compruebo si debo incluir en las mejores jugadas
			_view.checkForBestPlays();
			
			//Comprobar si el siguiente nivel es mayor que el nivel mximo disponible guardado, y si es mayor, actualizar
			checkForMaxLevelAvailable();
			
			//Una vez informado del cambio de nivel pasamos al siguiente nivel
			//Crear una actividad nueva para esto o mtodo, esto es temporal
			_level++;
			
			//Actualizamos el nivel actual
			MyApplication.setCurrentLevel(_level);
			
			startLevel();
		}
	}
    
    /**
     * Mtodo ejecutado cuando se pierde en un nivel. 
     */
    private void lostGame()
    {
   
    	MyApplication.getSoundManager().pausePlayer();
    	
    	//Pauso el contador del tiempo
    	_mustCount = false;
    	
    	//Inhabilito los controles de interaccin
    	_frmStartGame.disableControls();
    	
    	//Escribo el mensaje de partida pardida
    	_frmStartGame.setTextViewFinishedLevel("Has perdido");
    	//Hago visible el mensaje de partida perdida
        _frmStartGame.setTextViewVisibleFinishedLevel();
         
        updateScreen();
        goToSleep(2000);
        MyApplication.getSoundManager().playSound(MyApplication.Sounds.Goat);
        MyApplication.getSoundManager().playSound(MyApplication.Sounds.Kid_Laugh);
        goToSleep(2000);
        MyApplication.getSoundManager().playSound(MyApplication.Sounds.Goat);
        MyApplication.getSoundManager().playSound(MyApplication.Sounds.Kid_Laugh);
        	
        removeAllLettersToScreen();
        
        //Reseteo el Display
        _view.resetDisplay();
        
        //Escribo los puntos totales en el textview que muestra los puntos
        _frmStartGame.setTextViewPointsValue(String.valueOf(_view.getPointsValue()));
        //Hago visible el textView 
        _frmStartGame.setTextViewVisiblePoints();
       
        updateScreen();
        goToSleep(4000);
        	
    	//Compruebo si debo incluir en las mejores jugadas
		_view.checkForBestPlays();
		
		//Vamos a la pantalla del menu principal
		_frmStartGame.backToMainMenuScreen();
    }
    
    
    
    /**
     * Mtodo ejecutado cuando finalizamos el nivel 10, es decir, hemos llegado al final del juego. 
     */
    private void winGame()
    {
    	MyApplication.getSoundManager().pausePlayer();
    	
    	//Pauso el contador del tiempo
    	_mustCount = false;
    	
    	
    	//Inhabilito los controles de interaccin
    	_frmStartGame.disableControls();
    	
    	MyApplication.getSoundManager().startPlayerWinningSound();
    	
    	removeAllLettersToScreen();
    	
        //Reseteo el Display
        _view.resetDisplay();
    	
    	//Escribo el mensaje de victoria
    	_frmStartGame.setTextViewFinishedLevel("Enhorabuena!!");
    	//Hago visible el mensaje de victoria
        _frmStartGame.setTextViewVisibleFinishedLevel();
          
        updateScreen();
        goToSleep(3000);
        
        //Escribo los puntos totales en el textview que muestra los puntos
        _frmStartGame.setTextViewPointsValue(String.valueOf(_view.getPointsValue()));
        //Hago visible el textView 
        _frmStartGame.setTextViewVisiblePoints();
        
        updateScreen();
        goToSleep(70000);	
        
    	//Compruebo si debo incluir en las mejores jugadas
		_view.checkForBestPlays();
		
		//Vamos a la pantalla del menu principal
		_frmStartGame.backToMainMenuScreen();
    }
    
    
    
    /**
     * Inidica en la pantalla el comienzo de un nuevo nivel e inicia el bucle de ejecucin del juego. 
     */
    private void startLevel()
	{	  	
    	//Oculto el HUD
    	_frmStartGame.hideHUD();
    	//Oculto los controles informativos
    	_frmStartGame.setTextViewInvisible();
    	
    	goToSleep(700);
    	
    	//Escribo el nuevo nivel en el textView
    	_frmStartGame.setTextViewNewLevel(String.valueOf(_level));
    	//Hago visible el nuevo nivel
    	_frmStartGame.setTextViewVisibleNewLevel();
    	goToSleep(2500);
    	
    	//Oculto el nuevo nivel
    	_frmStartGame.setTextViewInvisibleNewLevel();
    	
    	goToSleep(1000);
    	
		_countTime = 0;
		_running = true;
		_isFinishedLevel = false;
        
    	//Habilito los controles de interaccin
    	_frmStartGame.enableControls();
        
		//Inicializo controles visuales
        _frmStartGame.setTextViewLevel(String.valueOf(_level));
        _frmStartGame.setTextViewTime(String.valueOf(_countTime));
        
        
		//Muestro el HUD
		_frmStartGame.showHUD();
		
		goToSleep(500);
		
    	//Reproduzco la msica ambiente
        MyApplication.getSoundManager().startPlayer();
        
    	//Reinicio contador del tiempo
        _mustCount = true;
		
		run();
		
	}
         
    
    
    /**
     * Imprime en el textview del HUD el nmero de segundos transcurridos. 
     */
    private void printInTextViewTime()
    {
    	_frmStartGame.setTextViewTime(String.valueOf(_countTime));
    }
    

   
   /**
    * Comprueba si el nuevo nivel que se inicia es superior al mayor nivel abierto, en tal caso, indica que el nuevo nivel
    * es el mayor nivel disponible. 
    */
   private void checkForMaxLevelAvailable()
   {
	  //El nivel que abrimos es el siguiente al actual, indicamos esto en una variable
	   int iLevelOpen = _level + 1;
	   
	   int imaxLevelAvailable = MyApplication.getSharedPreferencesManager().readIntFromSharedPrefences("Levels");
		
		//Significa que an no se ha indicado ningn nivel habilitado, por lo tanto, indicamos que el siguiente nivel
		//ha sido habilitado o abierto
		if(imaxLevelAvailable == -1)
			MyApplication.getSharedPreferencesManager().writeIntToSharedPreferences("Levels", iLevelOpen);
		//Hay un nivel mximo indicado, comparo con el actual, en caso de ser menor actualizo
		else
		{
			if(iLevelOpen > imaxLevelAvailable)
			{
				//Elimino el actual y en su lugar pongo el nuevo
				MyApplication.getSharedPreferencesManager().removeEntryToSharedPreferences("Levels");
				//Inserto el nuevo
				MyApplication.getSharedPreferencesManager().writeIntToSharedPreferences("Levels", iLevelOpen);
				
			}
		}
   }
   

   /**
    * Contabiliza el numero de letras en la pantalla una vez finalizado el nivel, restando un punto por cada letra.
    * @return: true si en la pantalla haba al menos una letra, false en caso contrario.
    */
   private boolean counterLettersLeft()
   {
	   int iNumLetters = 0;
	   
	   for(int index = (_view.listSpritesSize() -1); _view.listSpritesSize() > 0; index--)
	   {
		   iNumLetters++;
		   
		   //Indico el nmero actual de letras contabilizadas
		   _frmStartGame.setTextViewLettersLeftValue(String.valueOf(((iNumLetters) * -1)));
       	   //Elimino la letra de la pantalla asociada a index
		   _view.removeListSprites(index);
       	   //Resto un punto a los puntos totales
       	   _view.setPointsValue((_view.getPointsValue()-1));
      
       	   updateScreen();
       	   
       	   MyApplication.getSoundManager().playSound(MyApplication.Sounds.Peop);
       	   //Pauso para que de tiempo a visualizar el cambio
       	   goToSleep(500);
	   }

	   return (iNumLetters > 0);

   }
   
   
   /**
    * Elimina todas las letras de la pantalla, mostrando su borrado de forma progresiva. 
    */
   private void removeAllLettersToScreen()
   { 
	   for(int index = (_view.listSpritesSize() -1); _view.listSpritesSize() > 0; index--)
	   {
       	   //Elimino la letra de la pantalla asociada a index
		   _view.removeListSprites(index);  	 
      
       	   updateScreen();
       	   
       	   //Pauso para que de tiempo a visualizar el cambio
       	   goToSleep(150);
	   }
   }
   
   
   /**
    * Dibuja la pantalla con su contenido en el momento actual.
    */
   public void updateScreen()
   {              
	   SurfaceHolder suerfaceHolder = _view.getHolder();
	   Canvas c = null;
	     
	   try 
	   {       
	 	  //Utilizamos de forma exclusiva el recurso, para evitar su uso simultneo.
	 	  c = suerfaceHolder.lockCanvas();
	 	  synchronized (c) 
	       {	
	 		  _view.onDraw(c); 
	       }
	   }
	   catch(ConcurrentModificationException e )
	   {
		   Log.e(Tag, "Problemas de concurrencia al utilizar el objeto Canvas: " + e.getMessage());
	   } 
	   finally 
	   {
	 	  if (c != null) 
	 	  {
	 		  synchronized (c) 
	           {
	 			  suerfaceHolder.unlockCanvasAndPost(c);
	                       
	           }
	        }
	    }	    
   }
   
   
   /**
    * Manda al hilo actual a dormir los milisegundos indicados en time.
    * @param time: milisegundos que debe dormir el hilo.
    */
   private void goToSleep(long time)
   {
	   if(!_destroy)
	   {
	       try 
	       {
	    	   _isSleeping = true;
	    	   sleep(time);
	    	   _isSleeping = false;
	    	 
	       } 
	       catch (InterruptedException e) 
	       {
	    	   Log.d(Tag, "El hilo GameLoopThread ha sido despertado: " + e.getMessage());
	       }
	   }
   }
   
   public boolean isRunning()
   {
	   return _running;
   }
   
   public boolean isSleeping()
   {
	   return _isSleeping;
   }
   
}  