package com.example.game;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import android.util.AttributeSet;
import android.util.Log;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;


/**
 * Clase utilizada para pintar en la pantalla donde transcurre la partida, en ella dibujo el HUD, 
 * los agentes y los controles de interaccin. sta se encarga de pintar a los agetnes letras en cada instante
 * en la posicin que proceda.
 * 
 * @author Antonio Javier Garrido Rodrguez
 * Copyright 2014 Antonio Javier Garrido Rodrguez
 * 
 *  This file is part of Kamikawa.

    Kamikawa is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Kamikawa is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Kamikawa.  If not, see <http://www.gnu.org/licenses/>.
 */
public class GameView extends SurfaceView {
	
	private static final String Tag = GameView.class.getSimpleName();

	 private SurfaceHolder _holder;
	 
	 //Referencia al hilo con el bucle de ejecucin del juego.
	 private GameLoopThread _gameLoopThread;
	 //Lista de agentes
	 private List<Sprite> sprites = new ArrayList<Sprite>();
	 //Lista de agentes que estn formando una palabra y sern elminados
	 private List<Sprite> _listSpritesToRemove = new ArrayList<Sprite>();
	 //Almacena el tiempo del ltimo click sobre la pantalla
	 private long lastClick;
	 
	 //Estructura para almacenar las palabras de la gramtica inglesa
	  ArrayList<String> _englishGrammarWords = null;
	 
	 //Almacena los puntos acumulados en la partida
	 private long _Points = 0;
	 
	 //Almacena la palabra actual
	 private String _Word;
	 
	 //Indica si la palabra actual es correcta
	 private boolean _wordIsCorrect = false;
	 
	//Referencia a la actividad que inicia la creacin de la pantalla
	 FrmStartGame _frmStartGame = null;
	 
	 //Lista que utilizo como referencia para saber a cada nmero generado aleatoriamente qu letra le corresponde
	 private List<DataAgent> _listDataAgent;
	 	 
	 //Almacena el prximo agente que debe aparecer
	 private int _nextAgent;
	 
	 //Referencia a la clase que gestiona las colisiones entre agentes
	 private ManagerCollision _managerCollision;
	 
	 //Contador para comprobar las colisiones entre agentes en una determinada frecuencia
	 private int _frequencyCheckCollision;
	 
	 //Almacena la altura de los controles que estn en la parte inferior de la pantalla
	 private int _heightControlsBottom;
	 
	 

	 /**
	 * 
	 * Inicializo todos los datos necesarios
	 * 
	 * @param context
	 * @param attrs
	 */
	public GameView(Context context, AttributeSet attrs) 
	 {	
		super(context, attrs);
			
		//Inicializamos variables
		_listDataAgent = null;
    	_nextAgent = 0;
		
		//Creo la lista con datos de los agentes
		makeDataAgent();
		
		//Obtengo la referencia a la clase que gestiona las colisiones
		_managerCollision = MyApplication.getManagerCollision();
		
		//Establezco el tiempo mnimo entre colisiones para el nivel actual
		_managerCollision.setTimeLastCollision(MyApplication.getCurrentLevel());
		
		_frequencyCheckCollision = 0;
		
		//Palabra actual vaca
		_Word = "";
		//Obtengo la referencia a la actividad
		_frmStartGame = (FrmStartGame) this.getContext();
		
		//Obtengo la referencia a la lista de palabras de la gramtica inglesa
		_englishGrammarWords = MyApplication.getEnglishGrammarWords();
		
		//Instancio el hilo que controla el bucle de ejecucin del juego						
		_gameLoopThread = new GameLoopThread(this);
		
		//Inicialmente el hilo debe vivir
		_gameLoopThread.setDestroy(false);
		
		_holder = getHolder();
		_holder.addCallback(new SurfaceHolder.Callback() {
			 
        /* 
         * Mtodo ejecutado automticamente cuando se destruye la instancia de sta clase
         */
        @Override
        public void surfaceDestroyed(SurfaceHolder holder) 
        {
        	boolean retry = true;
            _gameLoopThread.setRunning(false);
            //Indico que el hilo debe finalizar
            _gameLoopThread.setDestroy(true);
            
            while (retry) 
            {
            	try 
                {
            		//Bloqueo el hilo para esperar su finalizacin
            		_gameLoopThread.join();
                    retry = false;
                } 
                catch (InterruptedException e) 
                {
                	Log.e(Tag, "Error en mtodo 'surfaceDestroyed' " + e.getMessage());
                }
            }
        }

        /* 
         * Mtodo ejecutado automticamente cuando se crea la instancia de sta clase
         */
        @Override
        public void surfaceCreated(SurfaceHolder holder) 
        {		
        	//Indico que el hilo debe ejecutarse
        	_gameLoopThread.setRunning(true);
            //Invoco mtodo que establece la configuracin de ejecucin del hilo
        	_gameLoopThread.setConfig();
            //Inicio la ejecucin del hilo
        	_gameLoopThread.start();
        	
        	//Obtengo la altura de los controles de la parte inferior de la pantalla
        	_heightControlsBottom = _frmStartGame.setParametersScreen();
        }

        @Override
        public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) 
        {

        }
    
			
		});	
	}
	

    /**
     * Mtodo con el cal pinto el fondo de pantalla ne blanco y dibujo los agentes letras. 
     *
     * @param canvas. Objeto canvas representa la pantalla del videojuego, lo que pintemos aparece en la pantalla del juego.
     */
    @Override
    protected void onDraw(Canvas canvas) 
    {	
    	//Pinto pantalla con el fondo blanco
    	canvas.drawColor(Color.WHITE);	
     
    	//Para cada sprite contenido en la lista, lo mando a dibujarse en la pantalla
    	for (Sprite sprite : sprites) 
    	{
    		sprite.onDraw(canvas);
    	}	
    	
    	
    	//Incrementamos la variable que controla la frecuencia de comprobacin de colisiones
    	_frequencyCheckCollision++;
    	
    	//As comprueba las colisiones cada 3 pasadas, y hace 10 pasadas cada segundo. Es interesante hacerlo as para que no haga tantas
    	// comprobaciones y no sobrecargar el juego de procesamiento.
    	if(_frequencyCheckCollision >= 3)
    	{
    		//Comprueba y gestiona las colisiones entre agentes
    		isCollisionSprites();
    		_frequencyCheckCollision = 0;
    	}
    	
		//Imprimo en la pantalla el nmero actual de agentes
		printInTextViewNumAgent();
    }
    
    
    
    /**
     * Mtodo ejecutado cada vez que hacemos un "touch" sobre la pantalla. 
     * Detecta si pulsamos sobre un agente, en tal caso, procesa la pulsacin.
     */
    @Override
    public boolean onTouchEvent(MotionEvent event) 
    {
    	
        //Si el juego no est pausado y est en modo "running"permitimos interaccin con la pantalla
        if(! _frmStartGame.isPausedGame() && _gameLoopThread.isRunning())
        {
	        //Haya pasado 300  milisegundo desde el ltimo click, as evitamos seleccionar dos agentes a la vez
	    	if (System.currentTimeMillis() - lastClick > 300) 
	        {
	            lastClick = System.currentTimeMillis();
	            
	            //Para utilizar la pantalla de forma exclusiva, y que lo agentes no cambien su posicin
		    	synchronized (getHolder()) 
		    	{
		    		//Para cada agente en la pantalla, comprueba si su posicin coincide con la posicin donde hemos pulsado.
		    		for (int i = sprites.size()-1; i >= 0; i--) 
		    		{
		                 Sprite sprite = sprites.get(i);
		                 if (sprite.isCollision(event.getX(),event.getY())) 
		                 {	
		                	 //Si es la primera vez que se marca, proceso al agente seleccionado
		                	 if(!_listSpritesToRemove.contains(sprite))
		                	 {
		                		 //Limito el nmero de agentes que podemos seleccionar para formar una nica palabra a 30
		                		 if(_listSpritesToRemove.size() < 30)
		                		 {
		                			 //Efecto sonoro de agente seleccionado
		                			 MyApplication.getSoundManager().playSound(MyApplication.Sounds.Click);
		                			 
			                		 //La palabra ha sido modificada, inicialmente no es correcta hasta que se compruebe y verifique
			                		 _wordIsCorrect = false;
			                		 
			                		 //Cambiamos el color del agente a rojo, indicando que ha sido seleccionado
			                		 sprite.changeColourToRed();
			                		 
			                		 //Nueva palabra, marcamos display rojo
			                		 _frmStartGame.setViewDisplayColorRed();
			                		 
			                		 //Pinto la letra en el Display
			                		 _Word = _Word + sprite.getLetra();
				                	 _frmStartGame.setTextViewDisplay(_Word);
				                	 
			                		 //La incluyo como letra de la palabra actual
				                	 _listSpritesToRemove.add(sprite);
			                		 
			                		 //Compruebo si la palabra es correcta
			                		 wordIsCorrect();
		                		 }
		                	 }
		                     
		                      //Una vez detectado un agente, toma solo uno, el de ms arriba
		                      break;
		                 }
		    		}
		    	}
	        }
        }
        
        //Para que maneje este evento slo este mtodo
        return true;
    }
    
    /**
     * Comprueba y gestiona las colisiones entre agentes.
     */
    private void isCollisionSprites()
    {
		
    	//Para todas las combinaciones posibles de agentes, compruebo si colisionan
    	for (int i = 0; i < sprites.size(); i++) 
		{
             Sprite spriteMain = sprites.get(i);
             
             for(int j = (i + 1); j < sprites.size(); j++)
             {
            	 Sprite spriteSecondary = sprites.get(j);
            	 
         		//Si ambos agentes estn colisionando
            	if(isCollisionSprite(spriteMain, spriteSecondary))
        		{   			       			
        			//Obtengo el instante de colisin
        			long timeCollision = System.currentTimeMillis();
        			        			
        			//Compruebo que sea una colisin permitida: que no est entre las 50 ltimas, y si est 
        			//que haya superado el tiempo mnimo de colisin establecido.
         			if(_managerCollision.isValidateCollision(i, j, timeCollision))
         			{
         				//Cambio la direccin de movimiento de uno de ellos
         				/*No puedo invertir la direccin de los dos sprites. Si slo colisionasen una vez sera lo ideal, porque ambos se repelen.
       				  	Pero si se vuelve a detectar una colisin mientras se estn alejando, invierten de nuevo su direccin, es decir, volveran
       				  	a acercarse y entraran en un bucle.
         				*/
         				spriteMain.changeDirection();
         				
         				//Indico la nueva colisin
         				_managerCollision.setNewCollision(i, j, timeCollision);
         			}
        		}
             }
		}
    }
    
    
    /**
     * 
     * Determina si los agentes spriteMain y spriteSecondary estn colisionando.
     * @param spriteMain: agente tomado como principal
     * @param spriteSecondary: agente tomado como secundario
     * @return true si los dos agentes estn colisionando, false en caso contrario.
     */
    private boolean isCollisionSprite(Sprite spriteMain, Sprite spriteSecondary)
    {
    	//Obtengo las coordenadas del agente secundario.
    	int xCoordinate =  spriteSecondary.getCoordinateX();
    	int yCoordinate =  spriteSecondary.getCoordinateY();
    	
    	//Obtengo el ancho y alto del polgono que lo representa
    	int width = spriteSecondary.getWidth();
    	int height = spriteSecondary.getHeight();
    	
    	//Compruebo si los dos agentes colisionan.
    	if( spriteMain.isCollision(xCoordinate, yCoordinate) || spriteMain.isCollision((xCoordinate + width), yCoordinate) || spriteMain.isCollision(xCoordinate, (yCoordinate + height)) || spriteMain.isCollision((xCoordinate + width), (yCoordinate + height)))
    		return true;
    	else
    		return false;
    }
    
    
    
    /**
     * Si la palabra actual es correcta: elimina los agentes que la representan, calcula los puntos conseguidos y los acumula al contador de puntos.
     * Si la palabra es incorrecta: Desmarca a los agentes que la representan.
     */
    public void checkWord()
    {
   	 	
    	//Si la palabra construida es correcta
    	if(_wordIsCorrect)
    	{
			 //Efecto sonoro de palabra correcta
			 MyApplication.getSoundManager().playSound(MyApplication.Sounds.Coins);
    		
    		//Eliminar agentes seleccionados
    		removeAgentToList();
    		
    		//Calcular los puntos conseguidos y acumularlos en el contador de puntos
    		calculatePoints();
    		
    		//Actualizo los puntos en la pantalla
    		_frmStartGame.setTextViewPoints(String.valueOf(_Points));
    		
    		//Decimos que la palabra vacia es falsa
    		_wordIsCorrect = false;
    		
    		//Pintamos Display en rojo
    		 _frmStartGame.setViewDisplayColorRed();
    		 
    	    //Inicializamos la palabra
    		_Word = "";
    		//Actualizamos Display
    	    _frmStartGame.setTextViewDisplay(_Word);
    	}
    	//Si la palabra no es vacia, palabra incorrecta, borramos palabra y desmarcamos agentes
    	else if(_Word.length() > 0)
    	{
			 //Efecto sonoro de borrar palabra
			 MyApplication.getSoundManager().playSound(MyApplication.Sounds.Eraser);
    		
    		//Reestablecemos el color por defecto
    		reestablishColour();
    		
    		//Desselecciona a los agentes marcados
    		//Vaciamos la lista de indices de sprites a eliminar
    		_listSpritesToRemove.clear();
    		
        	//Inicializamos la palabra
    		_Word = "";
    		//Actualizamos Display
        	_frmStartGame.setTextViewDisplay(_Word);

    	}
    }
    
   
    /**
     * Comprueba si la palabra actual es correcta, es decir, pertenece a la lista de palabras vlidas de la lengua inglesa.
     * En tal caso, indica que la palabra es correcta y pinta el Display verde.
     */
    public void wordIsCorrect()
    {
    	if(listContainWord(_Word))
    	{
    		_wordIsCorrect = true;
    		//La palabra actual es correcta,  pinta el Display en verde para informar de ello
    		_frmStartGame.setViewDisplayColorGreen();
    	}

    }
    

    /**
     * Elimina de la lista de agentes los indicados en la lista de sprites a eliminar.
     */
    public void removeAgentToList()
    {
		for (int i = 0; i < _listSpritesToRemove.size(); i++) 
			sprites.remove(_listSpritesToRemove.get(i));
		
		
		//Vaciamos la lista de indices de sprites a eliminar
		_listSpritesToRemove.clear();
    }
    
   
    /**
     * Calcula los puntos para la palabra actual, y los aade al total de puntos. 
     */
    public void calculatePoints()
    {
    	//Los puntos se calcula de forma exponencial en base 2, sera 2 elevado a la longitud de la palabra
    	int length = _Word.length();
    	 
    	_Points = _Points + (long) Math.pow(2, length);
    	
    }    
    
    
    /**
     * @return referencia al hilo que controla la ejecucin del juego.
     */
    public GameLoopThread getGameLoopThread()
    {
    	return _gameLoopThread;
    }

    
    /**
     * Para cada agente contenido en la lista temporal de agentes a eliminar, reestablece su color al color original. 
     */
    private void reestablishColour()
    {
		for (int i = 0; i < _listSpritesToRemove.size(); i++) 
		{
             _listSpritesToRemove.get(i).changeColourToDefault();
		}
    	
    }
    
    /**
     * Comprueba si la palabra word est contenida en la lista de palabras vlidas de la lengua inglesa.
     * @param word palabra a buscar en la lista.
     * @return true si la palabra pertenece a la lista, false en caso contrario.
     */
    private boolean listContainWord(String word)
    {
    	int result = searchWord(word, 0, _englishGrammarWords.size());
    	
    	if(result != -1)
    		return true;
    	else
    		return false;
    	
    }
    
    /**
     * Busca en la lista de palabras de la lengua inglesa la palabra word.
     * @param word palabra a buscar en la lista.
     * @param from inicio de la lista
     * @param to final de la lista
     * @return 0 si la palabra word pertenece a la lista, -1 en caso contrario.
     */
    private int searchWord(String word, int from, int to)
    {
    	if(to < from)
    		return -1;
    	
    	int middle = (from + to) / 2;
    	int result = word.compareTo(_englishGrammarWords.get(middle));
    	
    	if(result == 0)
    		return middle;
    	
    	if(result < 0)
    		return searchWord(word, from, middle-1);
    	else
    		return searchWord(word, middle+1, to);
    	
    }
    
    
    /**
     * Reordena la lista con las 10 mejores jugadas, empezando a reordenar por la posicin posIni, 
     * e insertando el nuevo valor iValuetoReOrder.
     * @param posIni posicin de la lista por la que debe empezar a reordenar.
     * @param iValuetoReOrder nuevo valor a incluir en la lista.
     */
    private void reOrder(int posIni, int iValuetoReOrder)
    {
    	int temp;
    	boolean bStop = false;
    	
    	while(posIni<11 && (!bStop))
    	{
    		temp = MyApplication.getSharedPreferencesManager().readIntFromSharedPrefences(String.valueOf(posIni));
    		//Quiere decir que no hay valor definido en dicha posicin, lo inserto y paro
    		if(temp == -1)
    		{
    			MyApplication.getSharedPreferencesManager().writeIntToSharedPreferences(String.valueOf(posIni), (int)iValuetoReOrder);
    			bStop = true;
    		}
    		//en la posicin actual hay un valor, lo cambio y sigo reordenando
    		else
    		{
				//Elimino el actual y en su lugar pongo el nuevo
				MyApplication.getSharedPreferencesManager().removeEntryToSharedPreferences(String.valueOf(posIni));
				//Inserto el nuevo
				MyApplication.getSharedPreferencesManager().writeIntToSharedPreferences(String.valueOf(posIni), (int)iValuetoReOrder);
				//El nuevo valor a insertar es el que haba en la posicin actual
				iValuetoReOrder = temp;
    		}
    		
    		posIni++;
    	}
    }
    
    /**
     * Comprueba si la puntuacin obtenida debe ser incluida entre las 10 mejores jugadas,
     * en tal caso, la incluye en la posicin que corresponda.
     */
    public void checkForBestPlays()
    {
    	boolean bStop = false;
    	int iValueReaded = -1;
    	int i = 1;
    	int iLastValueReaded = -1;
    	
    	while(i<11 && (!bStop))
    	{
    		iValueReaded = MyApplication.getSharedPreferencesManager().readIntFromSharedPrefences(String.valueOf(i));
    		
    		//Quiere decir que no hay valor definido en dicha posicin, si los puntos son positivos
    		//y distinto que el ltimo valor ledo (evitar valores repetidos) lo almaceno
    		if(iValueReaded == -1 && _Points > 0 && _Points != iLastValueReaded)
    		{
    			MyApplication.getSharedPreferencesManager().writeIntToSharedPreferences(String.valueOf(i), (int)_Points);
    			bStop = true;
    		}
    		//El valor es superior al actual y distinto que el ltimo valor ledo (evitar valores repetidos) lo almaceno
    		else if(iValueReaded > 0 && _Points > iValueReaded && _Points != iLastValueReaded)
    		{	
    			//Elimino el actual y en su lugar pongo el nuevo
    			MyApplication.getSharedPreferencesManager().removeEntryToSharedPreferences(String.valueOf(i));
    			//Inserto el nuevo
    			MyApplication.getSharedPreferencesManager().writeIntToSharedPreferences(String.valueOf(i), (int)_Points);
    			//Reordeno el resto de valores
    			reOrder((i+1), iValueReaded);
    			bStop = true;
    		}
    		//Posicin vaca y no hemos insertado, paramos
    		else if(iValueReaded == -1)
    			bStop = true;
    			
    		iLastValueReaded = iValueReaded;
    		i++;
    	}
    }
    
    /**
     * Inicializa el contenido del Display y lo pinta en rojo. 
     */
    public void resetDisplay()
    {
    	//Inicializamos la palabra
		_Word = "";
		//Actualizamos Display
    	_frmStartGame.setTextViewDisplay(_Word);
		//Pintamos Display en rojo
		 _frmStartGame.setViewDisplayColorRed();	
    	
    }
    
    /**
     * Devuelve los puntos actuales
     * @return long con los puntos actuales.
     */
    public long getPointsValue()
    {
    	return _Points;
    }
    
    
    /**
     * Establece los puntos actuales al valor points pasado por parmetro.
     * @param points nuevo valor para los puntos.
     */
    public void setPointsValue(long points)
    {
    	_Points = points;
    }
    
    
    /**
     * Crea a un nuevo agente. 
     */
    public void createAgent()
    {
		  createSprites();
		  
		  //Inmediatamente despus de crear el agente, obtenemos el siguiente que debe aparecer
		  prepareNextAgent();
		  
    }
    
    /**
     * Imprime en el TextView del HUD que indica el prximo agente cul es el prximo agente 
     */
    private void printInTextViewNextAgent()
    {
    	_frmStartGame.setTextViewNextAgent(_listDataAgent.get(_nextAgent).getStringAgent());
    }
    
    
    /**
     * Imprime en el TextView del HUD que indica el nmero agente, la cantidad de agentes en pantalla. 
     */
    private void printInTextViewNumAgent()
    {
    	_frmStartGame.setTextViewNumAgent(String.valueOf(sprites.size()));
    }
    
    
    /**
     * Crea el agente indicado en el atributo  _nextAgent.
     */
    private void createSprites() 
    {
       //Como ya sabemos la letra que tiene este sprite, en funcin a la letra (nmero) determinamos la imagen a cargar, y le pasamos
       //la letra que lo representa como parmetro
       sprites.add(createSprite(_listDataAgent.get(_nextAgent).getIdImg(), _listDataAgent.get(_nextAgent).getIdImgRed(), _listDataAgent.get(_nextAgent).getStringAgent()));     
    }
    
    /**
     * Crea al nuevo agente indicado por parmetros.
     * @param resouce: identificador de su imagen original.
     * @param resouce_Red: identificador de su imagen en rojo.
     * @param sLetraAgent: letra que representa el agente.
     * @return referencia a la instancia de un nuevo agente.
     */
    private Sprite createSprite(int resouce, int resouce_Red, String sLetraAgent) 
    {
        return new Sprite(this, resouce, resouce_Red, MyApplication.getCurrentLevel(), sLetraAgent);
    }
    
    
    /**
     * Determina cual es el siguiente agente que aparecer en pantalla. 
     */
    public void prepareNextAgent()
    {
     
        //Para generar nmero aleatorios y obener a travs de ste el prximo agente letra
        Random rnd = new Random(System.currentTimeMillis());
    	int iRandom = rnd.nextInt(1000);
        
        
        //45% de probabilidad para que sea vocal
       if(iRandom < 450)
    	   //Debemos generar una vocal, las vocales estn representadas por los nmeros del 0 al 4. 0 es a, y 4 u
    	   _nextAgent = rnd.nextInt(4);
    	   
       
       //55% probabilidad para que sea consonante
       else
       	   //Debemos generar una consonante, las consonantes estn representadas por los nmeros del 5 al 25. 5 es b, y 25 z
    	   _nextAgent = rnd.nextInt(20) + 5;   
       
       //Informa del prximo agente a mostrar, imprimiendolo en el textView
       printInTextViewNextAgent();
    }
    
    /**
     * Devuelve el tamao de la lista de agentes.
     * @return int nmero de agentes contenidos en la lista.
     */
    public int listSpritesSize()
    {
    	return sprites.size();
    }
    
    
    /**
     * Elimina el agente de la lista de agentes que ocupa la posicin indicada por index.
     * @param index: posicin del agente a eliminar.
     */
    public void removeListSprites(int index)
    {
    	sprites.remove(index);
    }
    
    /**
     * Devuelve la altura de los controles de la parte inferior de la pantalla.
     * 
     * @return int:  altura de los controles de la parte inferior de la pantalla.
     */
    public int getHeightControlsBottom()
    {
    	return _heightControlsBottom;
    }
    
    
    /**
     * Creo la lista que utilizo como referencia para saber a cada nmero generado aleatoriamente qu letra le corresponde. 
     */
    private void makeDataAgent()
    {
  	  _listDataAgent = new ArrayList<DataAgent>();
  	  DataAgent pDataAgent = null;
  	  //Debo insertar cada letra en la posicin correspondiente, primero vocales y luego consonantes, y la imagen asociada a cada letra
  	  //Inicialmente todos los agentes van a ser iguales
  	  pDataAgent = new DataAgent("a", R.drawable.a_blue, R.drawable.a_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("e", R.drawable.e_blue, R.drawable.e_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("i", R.drawable.i_blue, R.drawable.i_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("o", R.drawable.o_blue, R.drawable.o_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("u", R.drawable.u_blue, R.drawable.u_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("b", R.drawable.b_blue, R.drawable.b_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("c", R.drawable.c_blue, R.drawable.c_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("d", R.drawable.d_blue, R.drawable.d_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("f", R.drawable.f_blue, R.drawable.f_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("g", R.drawable.g_blue, R.drawable.g_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("h", R.drawable.h_blue, R.drawable.h_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("j", R.drawable.j_blue, R.drawable.j_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("k", R.drawable.k_blue, R.drawable.k_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("l", R.drawable.l_blue, R.drawable.l_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("m", R.drawable.m_blue, R.drawable.m_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("n", R.drawable.n_blue, R.drawable.n_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("p", R.drawable.p_blue, R.drawable.p_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("q", R.drawable.q_blue, R.drawable.q_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("r", R.drawable.r_blue, R.drawable.r_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("s", R.drawable.s_blue, R.drawable.s_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("t", R.drawable.t_blue, R.drawable.t_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("v", R.drawable.v_blue, R.drawable.v_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("w", R.drawable.w_blue, R.drawable.w_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("x", R.drawable.x_blue, R.drawable.x_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("y", R.drawable.y_blue, R.drawable.y_red);
  	  _listDataAgent.add(pDataAgent);
  	  
  	  pDataAgent = new DataAgent("z", R.drawable.z_blue, R.drawable.z_red);
  	  _listDataAgent.add(pDataAgent);	  	  	 	  	  	   	  	  
    }
}
