package com.example.game;

import java.util.HashMap;
import android.content.Context;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.SoundPool;
import android.util.Log;


/**
 * Clase utilizada para gestionar los sonidos de efectos y ambiental.
 * 
 * @author Antonio Javier Garrido Rodrguez
 * Copyright 2014 Antonio Javier Garrido Rodrguez
 * 
 *  This file is part of Kamikawa.

    Kamikawa is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Kamikawa is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Kamikawa.  If not, see <http://www.gnu.org/licenses/>.
 */
public class SoundManager 
{
	private static final String Tag = SoundManager.class.getSimpleName();
	
        private SoundPool _soundPool; 
        private HashMap<Integer, Integer> _soundPoolMap; 
        private AudioManager _audioManager;
        private Context _context;
        
        //Reproductor para la msica ambiental de fondo.
        private MediaPlayer _player;
        
        //Reproductor para la cancin del final del juego.
        private MediaPlayer _playerWinningSong;
        
        
        /**
         * Crea los dos reproductores utilizados durante el juego e inicializa valores de configuracin.
         * @param context
         */
		public void init(Context context)
        {
        	_context = context;
            _soundPool = new SoundPool(10, AudioManager.STREAM_MUSIC, 0); 
                  
            _soundPoolMap = new HashMap<Integer, Integer>();     
            
            _audioManager = (AudioManager)context.getSystemService(Context.AUDIO_SERVICE); 
            
            //Aado la msica de fondo al reproductor
            _player = MediaPlayer.create(context, R.raw.heythere);
            //Establezco reproduccin cclica.
            _player.setLooping(true);
            
            //Aado al reproductor la cancin ganadora
            _playerWinningSong = MediaPlayer.create(context, R.raw.winner);
        }
        
        
        /**
         * Aade el sonido SoundID a la lista de sonidos disponibles en la posicin Index.
         * @param Index: entero el cual indica en la posicin de la lista que ser aadido.
         * @param SoundID: entero que identifica el sonido que se desea aadir.
         */
        public void addSound(int Index, int SoundID)
        {
        	if(!_soundPoolMap.containsKey(Index))
            {
        		_soundPoolMap.put(Index, _soundPool.load(_context, SoundID, 1));
            }
        }
        
        
        /**
         * Reproduce el sonido de la lista de sonidos cuyo ndice sea index.
         * @param index: entero indicando el ndice del sonido que se desea reproducir
         */
        public void playSound(int index) 
        {       
        	//Compruebo que el sonido de efectos est activado
        	if(MyApplication.getSoundEffects())
        	{
            	if(_soundPoolMap.containsKey(index))
            	{
            		int streamVolume = _audioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
            		_soundPool.play((Integer) _soundPoolMap.get(index), streamVolume, streamVolume, 1, 0, 1f);
            	}
        	}
        }
        
        
        /**
         * Inicia la reproduccin de la msica ambiental de fondo. 
         */
        public void startPlayer()
        {
        	//Compruebo que el sonido de ambiente est activado
        	if(MyApplication.getSoundEnvironment())
        	{
            	int streamVolume = _audioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
            	_player.setVolume(streamVolume, streamVolume);
            	
            	try
            	{
            		_player.start();
            	}
            	catch(IllegalStateException ex)
            	{
            		Log.e(Tag, "Error en mtodo 'startPlayer'. " + ex.getMessage());
            	}
        	}
        }
        
        /**
         * Inicia la reproduccin de la msica de final del juego. 
         */
        public void startPlayerWinningSound()
        {
        	//Compruebo que el sonido de ambiente est activado
        	if(MyApplication.getSoundEnvironment())
        	{
            	int streamVolume = _audioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
            	_playerWinningSong.setVolume(streamVolume, streamVolume);
            	
            	try
            	{
            		_playerWinningSong.start();
            	}
            	catch(IllegalStateException ex)
            	{
            		Log.e(Tag, "Error en mtodo 'startPlayerWinningSound'. " + ex.getMessage());
            	}
        	}
        }
        
        /**
         * Pausa la reproduccin de la msica ambiental de fondo. 
         */
        public void pausePlayer() 
        {   
        	//Compruebo que el sonido de ambiente est activado
        	if(MyApplication.getSoundEnvironment())
        	{
            	try
            	{
            		_player.pause();
            	}
            	catch(IllegalStateException ex)
            	{
            		Log.e(Tag, "Error en mtodo 'pausePlayer'. " + ex.getMessage());
            	}
        		
        	}
        }
        
        
        /**
         * Pausa la reproduccin de la msica de final del juego. 
         */
        public void pausePlayerWinningSound() 
        {   
        	//Compruebo que el sonido de ambiente est activado
        	if(MyApplication.getSoundEnvironment())
        	{
            	try
            	{
            		_playerWinningSong.pause();
            		
            	}
            	catch(IllegalStateException ex)
            	{
            		Log.e(Tag, "Error en mtodo 'pausePlayerWinningSound'. " + ex.getMessage());
            	}
        	}
        }
        
        
        /**
         * Para la reproduccin de la msica de fondo y libera los recursos asociados al reproductor.
         */
        public void stopPlayer() 
        {    
        	//Compruebo que el sonido de ambiente est activado
        	if(MyApplication.getSoundEnvironment())
        	{
            	try
            	{
                    _player.stop();
                    _player.release();
                    
                  //Inicializo el reproductor para una futura reproduccin
                    _player = MediaPlayer.create(_context, R.raw.heythere);
            	}
            	catch(IllegalStateException ex)
            	{
            		Log.e(Tag, "Error en mtodo 'stopPlayer'. " + ex.getMessage());
            	}
        		
        	}
        }
        
        /**
         * Para la reproduccin de la msica de final del juego y libera los recursos asociados al reproductor.
         */
        public void stopPlayerWinningSound() 
        {    
        	//Compruebo que el sonido de ambiente est activado
        	if(MyApplication.getSoundEnvironment())
        	{
            	try
            	{
            		
            		_playerWinningSong.stop();
            		_playerWinningSong.release();
            		   	
                    //Inicializo el reproductor para una futura reproduccin
                    _playerWinningSong = MediaPlayer.create(_context, R.raw.winner);
            	}
            	catch(IllegalStateException ex)
            	{
            		Log.e(Tag, "Error en mtodo 'stopPlayerWinningSound'. " + ex.getMessage());
            	}
        		
        	}
        }
        
        
        /** 
         * Comprueba si los reproductores estn sonando, en tal caso, los para.
         */
        public void checkPlayers() 
        {    
        	//Compruebo que el sonido de ambiente est activado
        	if(MyApplication.getSoundEnvironment())
        	{
            	if(_player.isPlaying())
            		pausePlayer();
            	
            	//El reproductor de la musica final no es cclico, por ello, debo pararlo y reiniciarlo
            	if(_playerWinningSong.isPlaying())
            		stopPlayerWinningSound();
        	}
        }
}